// Scroll Traces
// $Author: Jeffrey J Weimer$
// SVN History: $Revision: 1607 $ on $Date: 2011-06-15 15:20:08 -0700 (Wed, 15 Jun 2011) $

#pragma rtGlobals=1
#pragma IgorVersion=6.20
#pragma version=1.5

#pragma IndependentModule=ScrollTraces

StrConstant thePackage="ScrollTraces"
StrConstant thePackageFolder="root:Packages:ScrollTraces"
StrConstant theProcedureFile = "ScrollTraces.ipf"
StrConstant thePackageInfo = "A control panel to scroll through multiple traces in a graph window"
StrConstant thePackageAuthor = "Jeffrey J Weimer"
Constant thePackageVersion = 1.5
Constant hasHelp = 1
Constant removable = 1

// Panel Size Parameters

Constant ndepth = 120
Constant nwidth = 255
Constant cdepth = 20
Constant cwidth = 255

// menu

Menu "Graph", hideable
	Submenu "Panels"
		"Scroll Traces", /Q, SetupScrollTracesPanel()
	end
end

// Use PackageTools

Function Initialize()

	string theCmd
	sprintf theCmd, "ProcGlobal#PackageExists(\"%s\")", thePackage
	Execute/Q/Z theCmd
	NVAR V_exists
	if (NVAR_exists(V_exists))
		sprintf theCmd,"\"%s\"", thePackage
		sprintf theCmd, "%s,folder=\"%s\"", theCmd, thePackageFolder
		sprintf theCmd, "%s,file=\"%s\"", theCmd, theProcedureFile
		sprintf theCmd, "%s,info=\"%s\"", theCmd, thePackageInfo
		sprintf theCmd "%s,author=\"%s\"", theCmd, thePackageAuthor
		sprintf theCmd, "%s,version=%f", theCmd, thePackageVersion
		sprintf theCmd, "%s,hasHelp=%d", theCmd, hasHelp
		switch(V_exists)
			case 0:
				sprintf theCmd, "ProcGlobal#PackageSetup(%s)", theCmd
				break
			case 1:
				sprintf theCmd, "ProcGlobal#PackageUpdate(%s)", theCmd
				break
		endswitch
		Execute/Q/Z theCmd
	endif
	return 0
end

// Preferred Access Point from External Calls
// ScrollTraces#ShowPanel([activate,collapse])

Function ShowPanel([activate,collapse])
	variable activate, collapse

	STRUCT WMCheckboxAction cba
	STRUCT WMButtonAction ba
	
	activate = ParamIsDefault(activate) ? 0 : 1
	collapse = ParamIsDefault(collapse) ? 0 : 1
	
	SetupScrollTracesPanel()
	if (activate)
		cba.eventCode=2
		TurnOnTraceScrollMode(cba)
	endif
	
	if (collapse)
		ba.eventCode=2
		CloseScrollTracePanel(ba)	
	endif
	
	return 0
end

// setup the scroll trace package

Function SetupScrollTracesPanel()

	// create data folder

	if (!DataFolderExists(thePackageFolder))
		DFREF cdf = GetDataFolderDFR()
		SetDataFolder root:
		NewDataFolder/O/S Packages
		NewDataFolder/S $thePackageFolder
		variable/G moviemode=0
		string/G currWin="", plugIns=""
		SetDataFolder cdf
	endif
	
	Initialize()
	
	// create or show panel
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin
	
	// cannot attach a new panel while in movie mode
	
	if (mm)
		DoWindow/F $cw
		return 0
	endif
	
	// panel attached according to front window name
	
	string fWin = WinName(0,1,1), dfWin
	dfWin = thePackageFolder + ":" + fWin
	
	if (!DataFolderExists(dfWin))
		MakeScrollTracesPanel(fWin)
	endif
	
	return 0
	
end

// make the panel

Function MakeScrollTracesPanel(fWin)
	string fWin
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin
	
	cw = fWin
	mm = 0
		
	// create globals for new panel
	
	DFREF cdf = GetDataFolderDFR()
	
	SetDataFolder $thePackageFolder	
	NewDataFolder/S $cw
	variable/G active=0, tracenumber=0, speed=1, loopmode=0, scalemode=0, externalmode=0, keyboard=1
	string/G panelName
	make/n=1 scrt_frozen=NaN
	
	NVAR ac = active
	NVAR tn = tracenumber
	NVAR sp = speed
	NVAR lm = loopmode
	NVAR sm = scalemode
	NVAR xm = externalmode
	NVAR kb = keyboard
	SVAR pn = panelName
	
	ac = 0
	tn = 0
	sp = 1
	lm = 0
	sm = 1
	xm = 0
	kb = 1
	sprintf pn, "ScrollTracesPanel_%s", cw
	
	// return to current data folder
	
	SetDataFolder cdf
	
	// set up and attach panel to current graph
	
	DoWindow/F $cw
	
	NewPanel/W=(0,0,nwidth,ndepth)/N=$pn/K=2/EXT=2/HOST=# as "Scroll Traces"
	ModifyPanel noEdit=1, fixedsize=1
	SetDrawLayer UserBack
	DrawLine 22,20,242,20

	Button XtendD,pos={4,5},size={15,20},proc=XtendDown,title="v", userData="opened"
	
	Slider slidetraces pos={30,10}, size={nwidth-60,5}, vert=0,side=0, limits={0,1,1}, proc=SlideThruTraces, variable=tn

	CheckBox onoff,pos={6,33},size={16,14},proc=TurnOnTraceScrollMode,title="",value= 0

	Button first,pos={24,30},size={20,20},disable=2,proc=StepToTrace,title="|<",font="Arial"
	Button previous,pos={49,30},size={20,20},disable=2,proc=StepToTrace,title="<",font="Arial"
	Button next,pos={120,30},size={20,20},disable=2,proc=StepToTrace,title=">",font="Arial"
	Button last,pos={145,30},size={20,20},disable=2,proc=StepToTrace,title=">|",font="Arial"

	SetVariable gototrace,pos={75,31},size={40,18},disable=2,proc=GoToTraceNumber,title=" "
	SetVariable gototrace,font="Arial",limits={0,89,0},value=tn

	Slider speed,pos={51,73},size={75,6},disable=2, variable=sp
	Slider speed,limits={1,20,1},value=1,side= 0,vert= 0

	CheckBox loop,pos={92,54},size={42,14},disable=2,proc=SetLoopMode,title="\\f01l\\f00oop",font="Arial"
	CheckBox loop,value= 0,mode=1

	CheckBox once,pos={40,54},size={44,14},disable=2,proc=SetLoopMode,title="\\f01o\\f00nce",font="Arial"
	CheckBox once,value= 1,mode=1

	Button LoopForward,pos={142,55},size={25,20},disable=2,proc=DoLoop,title=">>",font="Arial"
	Button LoopBack,pos={8,50},size={25,20},disable=2,proc=DoLoop,title="<<",font="Arial"

	CheckBox scale_a,pos={180,30},size={29,14},disable=2,proc=SetScaleMode,title="\\f01a\\f00ll",font="Arial"
	CheckBox scale_a,value= 1,mode=1
	CheckBox scale_c,pos={180,46},size={52,14},disable=2,proc=SetScaleMode,title="\\f01c\\f00urrent",font="Arial"
	CheckBox scale_c,value= 0,mode=1
	CheckBox scale_h,pos={180,62},size={50,14},disable=2,proc=SetScaleMode,title="\\f01h\\f00old",font="Arial"
	CheckBox scale_h,value= 0,mode=1
	CheckBox scale_f,pos={180,78},size={50,14},disable=2,proc=SetScaleMode,title="\\f01f\\f00reeze",font="Arial"
	CheckBox scale_f,value= 0,mode=1
	CheckBox scale_r,pos={180,94},size={50,14},disable=2,proc=SetScaleMode,title=">|\\f01r\\f00|<",font="Arial", value= 0

	CheckBox keyboard, pos={6,85}, title="@", font="Arial", fsize=10, disable=2, value=1, proc=ToggleKeyboard
	
	PopupMenu external, pos={45,90}, mode=1, title="", font="Arial", value="-;x;!;", fsize=0, disable=2, proc=ChangeExternalMode

	SetVariable fhtrace, pos={nwidth-30,65},size={25,15}, font="Arial", fsize=8, limits={0,89,0}, value=tn, disable=2, title=" "
	
	Button RemoveMe, pos={120,90}, size={20,20}, title="X", font="Arial", fsize=10, proc=RemoveScrollTraces
	Button GetHelp, pos={145,90}, size={20,20}, title="?", font="Arial", fsize=10, proc=GetHelp
	
	string vers
	sprintf vers "v %g", thePackageVersion
	SetDrawLayer UserBack
	DrawLine 174,30,174,110
	SetDrawEnv fsize= 8;
	DrawText 5,113,vers
	sprintf vers "%d", ItemsInList(scrtTraceNameList())
	SetDrawEnv fsize= 8;
	DrawText nwidth-15,15, vers
	
	if (cmpstr(IgorInfo(2),"Macintosh")==0)
		Button ClosePanel, pos={228,30}, size={15,15}, fsize=8, title="p", proc=CloseScrollTracePanel
	endif
	
	// activate the basic window hooks
	
	SetWindowHooks(0)

	return 0
end

// slide through traces

Function SlideThruTraces(sa) : SliderControl
	STRUCT WMSliderAction &sa

	switch( sa.eventCode )
		case -1: // kill
			break
		default:
			if( sa.eventCode & 1 ) // value set
				DoAtTheTrace(sa.curval)
			endif
			break
	endswitch

	return 0
End

// turn on/off the panel operation

Function TurnOnTraceScrollMode(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	
	// cannot activate a panel when another one is in movie mode
	
	if (mm)
		return 0
	endif

	SVAR cw = $(thePackageFolder + ":currWin")
	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR ac = cwDFR:active
	NVAR tn = cwDFR:tracenumber
	NVAR sp = cwDFR:speed
	NVAR lm = cwDFR:loopmode
	NVAR sm = cwDFR:scalemode
	NVAR xm = cwDFR:externalmode
	NVAR kb = cwDFR:keyboard
	SVAR pn =  cwDFR:panelName

	switch( cba.eventCode )
		case 2: // mouse up
			variable nt = ItemsInList(scrtTraceNameList())
			ac = cba.checked
			tn = 0
			sp = 1
			lm = 0
			sm = 1
			xm = 0
			kb = 1
			HideTools/W=$cw
//			SetActiveSubWindow $(cw + "#" + pn)
			if (cba.checked)
				if (nt==0)
					CheckBox $cba.ctrlName win=$(cw + "#" + pn), value = 0
					break
				endif
				nt += -1
				string theOne = StringFromList(0,scrtTraceNameList())
				SetVariable gotoTrace win=$(cw + "#" + pn), disable=0,limits={0,nt,0}
				Slider slidetraces win=$(cw + "#" + pn), disable=0, limits={0,nt,1}
				Button first win=$(cw + "#" + pn), disable=0
				Button previous win=$(cw + "#" + pn), disable=0
				Button next win=$(cw + "#" + pn), disable=0
				Button last win=$(cw + "#" + pn), disable=0
				Button LoopForward win=$(cw + "#" + pn), disable=0
				Button LoopBack win=$(cw + "#" + pn), disable=0
				Slider speed win=$(cw + "#" + pn), disable=0, value=1
				CheckBox loop win=$(cw + "#" + pn), disable=0, value=0
				CheckBox once win=$(cw + "#" + pn), disable=0, value=1
				Checkbox scale_a win=$(cw + "#" + pn), disable=0, value=1
				Checkbox scale_c win=$(cw + "#" + pn), disable=0, value=0
				Checkbox scale_h win=$(cw + "#" + pn), disable=0, value=0
				Checkbox scale_f win=$(cw + "#" + pn), disable=0, value=0
				Checkbox scale_r win=$(cw + "#" + pn), disable=0, value=0
				Checkbox keyboard win=$(cw + "#" + pn), disable=0, value=1
				PopupMenu external win=$(cw + "#" + pn), disable=0
				SetVariable fhtrace win=$(cw + "#" + pn), disable=0,limits={0,nt,0}
				ModifyGraph/W=$cw hideTrace=2
				ModifyGraph/W=$cw hideTrace($theOne)=0
			else
				SetVariable gotoTrace win=$(cw + "#" + pn), disable=2
				Slider slidetraces win=$(cw + "#" + pn), disable=2
				Button first win=$(cw + "#" + pn), disable=2
				Button previous win=$(cw + "#" + pn), disable=2
				Button next win=$(cw + "#" + pn), disable=2
				Button last win=$(cw + "#" + pn), disable=2
				Button LoopForward win=$(cw + "#" + pn), disable=2
				Button LoopBack win=$(cw + "#" + pn), disable=2
				Slider speed win=$(cw + "#" + pn), disable=2, value=1
				CheckBox loop win=$(cw + "#" + pn), disable=2, value=0
				CheckBox once win=$(cw + "#" + pn), disable=2, value=1
				Checkbox scale_a win=$(cw + "#" + pn), disable=2, value=1
				Checkbox scale_c win=$(cw + "#" + pn), disable=2, value=0
				Checkbox scale_h win=$(cw + "#" + pn), disable=2, value=0
				Checkbox scale_f win=$(cw + "#" + pn), disable=2, value=0
				Checkbox scale_r win=$(cw + "#" + pn), disable=2, value=0
				Checkbox keyboard win=$(cw + "#" + pn), disable=2, value=1
				PopupMenu external win=$(cw + "#" + pn), disable=2
				SetVariable fhtrace win=$(cw + "#" + pn), disable=2
				ModifyGraph/W=$cw hideTrace=0
			endif
//			SetActiveSubWindow ##
			break
	endswitch

	return 0
end
// extend the panel down
 
Function XtendDown(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode

	// cannot activate a panel when another one is in movie mode
	
	if (mm)
		return 0
	endif

	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pn = cwDFR:panelName
 
	switch( ba.eventCode )
		case 2: // mouse up
			strswitch(ba.userData)
				case "closed":
					Button XtendD win=$(cw + "#" + pn), title="v", userData="opened"
					MoveSubWindow/W=$(cw + "#" + pn) fnum=(0,0,nwidth,ndepth)
					break
				case "opened":
					Button XtendD win=$(cw + "#" + pn), title=">", userData="closed"
					MoveSubWindow/W=$(cw + "#" + pn) fnum=(0,0,cwidth,cdepth)
					break
			endswitch
			break
	endswitch
	return 0
end

// set the graph active to recognize it has an attached panel

Function SetWindowHooks(how)
	variable how

//	NVAR mm = $(thePackageFolder + ":moviemode")
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pn = cwDFR:panelName
	
	switch(how)
		case -1:	// clear
			SetWindow $cw hook(ScrTHook)=$""
			SetWindow $cw hook(ScrTKeyboard)=$""
			SetWindow $cw hook(ScrTMovieMode)=$""
			SetWindow $(cw + "#" + pn) hook(ScrTHook)=$""
			SetWindow $(cw + "#" + pn) hook(ScrTKeyboard)=$""
			SetWindow $(cw + "#" + pn) hook(ScrTMovieMode)=$""
			break			
		case 0:	// set active non-movie mode
			SetWindow $cw hook(ScrTHook)=ScrollTracesHook
			SetWindow $cw hook(ScrTKeyboard)=ScrollTracesKeyboardHook
			SetWindow $cw hook(ScrTMovieMode)=$""
			SetWindow $(cw + "#" + pn) hook(ScrTHook)=ScrollTracesHook
			SetWindow $(cw + "#" + pn) hook(ScrTKeyboard)=ScrollTracesKeyboardHook
			SetWindow $(cw + "#" + pn) hook(ScrTMovieMode)=$""
			break
		case 1:	// set active movie mode
			SetWindow $cw hook(ScrTHook)=$""
			SetWindow $cw hook(ScrTKeyboard)=$""
			SetWindow $cw hook(ScrTMovieMode)=ScrollTracesMovieModeHook
			SetWindow $(cw + "#" + pn) hook(ScrTHook)=$""
			SetWindow $(cw + "#" + pn) hook(ScrTKeyboard)=$""
			SetWindow $(cw + "#" + pn) hook(ScrTMovieMode)=ScrollTracesMovieModeHook
			break
	endswitch
	
	return 0
end

// Button Action to Remove ScrollTraces

Function RemoveScrollTraces(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pn = cwDFR:panelName

	string theCmd = "PackageRemove(\"ScrollTraces\")"
	
	switch( ba.eventCode )
		case 2:
			if (mm)
				return -1
			endif
			KillWindow $(cw + "#" + pn)
			LoadScrollTracesPlugIns(0)			
			Execute/P/Q/Z theCmd
			break
	endswitch
	return 0
end

// Button Action to Get Help

Function GetHelp(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode

	string theCmd = "PackageHelp(\"ScrollTraces\")"

	switch( ba.eventCode )
		case 2:
			if (mm)
				return -1
			endif
			Execute/P/Q/Z theCmd
			break
	endswitch

	return 0
End

// change external function mode

Function ChangeExternalMode(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR xm = cwDFR:externalmode

	switch( pa.eventCode )
		case 2: // mouse up
			xm = pa.popNum-1
			LoadScrollTracesPlugIns(xm)
			break
	endswitch

	return 0
End

// load/unload plugins

Function LoadScrollTracesPlugIns(how)
	variable how
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pl = $(thePackageFolder + ":plugIns")
	NVAR tn = cwDFR:tracenumber

	string dirpath="", plugInFiles="", theOne="", plugIns="", theCmd=""
	variable ic=0
	
	switch(how)
		case 0:
		case 1:
			if (strlen(pl)==0)
				break
			endif
			plugInFiles = FunctionList("ScrTracesPI_*",";","WIN:[ProcGlobal]")
			if (how==1)
				plugInFiles = RemoveFromList("ScrTracesPI_Default",plugInFiles)
			endif
			ic = 0
			do
				theCmd=StringFromList(ic,plugInFiles)
				if (strlen(theCmd)==0)
					break
				endif
				theOne = StringFromList(tn,scrtTraceNameList())
				sprintf theCmd, "ProcGlobal#%s(1,\"%s\",\"%s\",%d)", theCmd, cw, theOne, tn
				Execute/Q/Z theCmd
				ic += 1
			while(1)
			ic = 0
			do
				theOne = StringFromList(ic,pl)
				if (strlen(theOne)==0)
					break
				endif
				sprintf theCmd, "DELETEINCLUDE \"%s\"", theOne
				Execute/Q/Z/P theCmd
				sprintf theCmd, "CloseProc/NAME=\"%s.ipf\"", theOne
				Execute/Q/Z/P theCmd
				ic +=1
			while(1)
			pl = ""
			break
		case 2:
			theOne = GetIndependentModuleName() + "#MakeScrollTracesPanel"
			theCmd = "setigoroption independentmoduledev=1"
			Execute/Q/Z theCmd
			dirpath = ParseFilePath(1,FunctionPath(theOne),":",1,0) 
			theCmd = "setigoroption independentmoduledev=0"
			Execute/Q/Z theCmd		
			if (strlen(dirpath)==0)
				break
			endif
			dirpath += "ScrTraces PlugIns"
			NewPath/O/Q ScrollTracesPlugIns, dirpath
			plugInFiles = IndexedFile(ScrollTracesPlugIns,-1,"????")
			ic = 0
			do
				theOne = ParseFilePath(3,StringFromList(ic,plugInFiles),";",0,0)
				if (strlen(theOne)==0)
					break
				endif
				// remove another extension in case windows alias
				theOne = ParseFilePath(3,theOne,";",0,0)
				sprintf theCmd, "INSERTINCLUDE \"%s\"", theOne
				Execute/Q/Z/P theCmd
				plugIns += theOne + ";"
				ic +=1
			while(1)
			Execute/Q/P/Z "COMPILEPROCEDURES "
			KillPath/Z ScrollTracesPlugIns
			pl = plugIns
			ic = 0
			do
				theCmd=StringFromList(ic,pl)
				if (strlen(theCmd)==0)
					break
				endif
				theOne = StringFromList(tn,scrtTraceNameList())
				sprintf theCmd, "ProcGlobal#%s(0,\"%s\",\"%s\",%d)", theCmd, cw, theOne, tn
				Execute/Q/Z/P theCmd
				ic += 1
			while(1)
			break
	endswitch
	Execute/Q/Z/P "COMPILEPROCEDURES "
	
	return 0
end

// close the panel (Macintosh only)

Function CloseScrollTracePanel(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode

	switch(ba.eventCode)
		case 2:		// button down
			if (mm)
				break
			endif
			CleanUpandKill()
			break	
	endswitch
	return 0
end

// trace name list excluding special scrt_frozen

Function/S scrtTraceNameList()

	string theList=RemoveFromList("scrt_frozen",TraceNameList("",";",1))

	return (theList)
end

// advance thru traces forward or backward via buttons

Function StepToTrace(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	
	if (mm)
		return 0
	endif

	SVAR cw = $(thePackageFolder + ":currWin")
	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR tn = cwDFR:tracenumber
	
	switch( ba.eventCode )
		case 2:
			strswitch(ba.ctrlName)
				case "first":
					tn = 0
					break
				case "next":
					tn += 1
					break
				case "last":
					tn = inf
					break
				case "previous":
					tn += -1
					break					
			endswitch
			STRUCT WMSetVariableAction sva
			sva.eventCode=99
			GoToTraceNumber(sva)
			break
	endswitch
	
	return 0
end

// go to a particular trace number using buttons, setvariable input, or movie input

Function GoToTraceNumber(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR tn = cwDFR:tracenumber
	NVAR lm = cwDFR:loopmode

	variable nt
	
	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
			DoAtTheTrace(sva.dval)
			break
		case 99: // left/right cursor key or movie mode
			nt = ItemsInList(scrtTraceNameList())
			if (tn < 0)
				tn = lm==0 ? 0 : nt-1
			endif
			if (tn >= nt)
				tn = lm==0 ? nt-1 : 0
			endif
			DoAtTheTrace(tn)
			break
	endswitch
	
	return 0
end

Function DoAtTheTrace(theN)
	variable theN
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR tn = cwDFR:tracenumber
	NVAR xm = cwDFR:externalmode
	
	string theOne = StringFromList(theN,scrtTraceNameList()), theFList="", theCmd=""
	SetScaleState()
	ModifyGraph/W=$cw hideTrace($theOne)=0
	tn = theN
	
	switch(xm)
		case 0:
			break
		case 1:
			sprintf theCmd, "ProcGlobal#ScrTracesPI_Default(0,\"%s\",\"%s\",%d)", cw, theOne, tn
			Execute/Q/Z theCmd
			break
		case 2:
			theFList = FunctionList("ScrTracesPI_*",";","WIN:[ProcGlobal]")
			theN = 0
			do
				theCmd=StringFromList(theN,theFList)
				if (strlen(theCmd)==0)
					break
				endif
				sprintf theCmd, "ProcGlobal#%s(0,\"%s\",\"%s\",%d)", theCmd, cw, theOne, tn
				Execute/Q/Z theCmd
				theN += 1
			while(1)
			break
	endswitch
	DoUpdate/W=$cw
	return 0
	
end

// turn on/off window hook functions for keyboard input

Function ToggleKeyboard(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR kb = cwDFR:keyboard

	switch( cba.eventCode )
		case 2: // mouse up
			kb = cba.checked
			break
	endswitch
	return 0
end

// select how the ordinate is to be scaled using the panel checkboxes

Function SetScaleMode(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR sm = cwDFR:scalemode
	SVAR pn = cwDFR:panelName
	NVAR tn = cwDFR:tracenumber
	wave sctf = cwDFR:scrt_frozen
	
	string theOne=""
	variable sf, fhtn = tn

	switch( cba.eventCode )
		case 99: // manual change
			Checkbox $cba.ctrlName win=$(cw + "#" + pn), value=1
		case 2: // mouse up
			ControlInfo/W=$(cw + "#" + pn) scale_r
			sf = V_value == 0 ? 1 : -1
			strswitch(cba.ctrlName)
				case "scale_a":
					Checkbox scale_c win=$(cw + "#" + pn), value=0
					Checkbox scale_h  win=$(cw + "#" + pn), value=0
					Checkbox scale_f  win=$(cw + "#" + pn), value=0
					SetVariable fhtrace win=$(cw + "#" + pn), value=tn
					removefromgraph/W=$(cw)/Z scrt_frozen	
					sm = 1*sf
					break
				case "scale_c":
					Checkbox scale_a win=$(cw + "#" + pn), value=0
					Checkbox scale_h  win=$(cw + "#" + pn), value=0
					Checkbox scale_f  win=$(cw + "#" + pn), value=0
					SetVariable fhtrace win=$(cw + "#" + pn), value=tn
					sm = 2*sf
					break
				case "scale_h":
					Checkbox scale_c  win=$(cw + "#" + pn), value=0
					Checkbox scale_a  win=$(cw + "#" + pn), value=0
					Checkbox scale_f  win=$(cw + "#" + pn), value=0
					SetVariable fhtrace win=$(cw + "#" + pn), value=_NUM:fhtn
					removefromgraph/W=$(cw)/Z scrt_frozen	
					sm = 3*sf
					break
				case "scale_f":
					Checkbox scale_c  win=$(cw + "#" + pn), value=0
					Checkbox scale_h  win=$(cw + "#" + pn), value=0
					Checkbox scale_a  win=$(cw + "#" + pn), value=0
					SetVariable fhtrace win=$(cw + "#" + pn), value=_NUM:fhtn
					removefromgraph/W=$(cw)/Z scrt_frozen	
					theOne = StringFromList(tn,scrtTraceNameList())
					duplicate/o TraceNametoWaveRef(cw,theOne) sctf
					if (WaveExists(XWaveRefFromTrace(cw,theOne)))
						wave wx = XWaveRefFromTrace(cw,theOne)
						appendtograph sctf vs wx
					else
						appendtograph sctf
					endif
					sm = 4*sf
					break
				case "scale_r":
					Checkbox scale_r  win=$(cw + "#" + pn), value=(sm>0)
					sm*=-1
					break
			endswitch
			SetScaleState()
			break
	endswitch
	
	return 0
end

// set how the ordinate is to be scaled (while advancing thru the traces)

Function SetScaleState()

	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR sm = cwDFR:scalemode
	NVAR tn = cwDFR:tracenumber
	
	string theOne
	variable as=1
	
	if (sm<0)
		as=2
	endif

	switch(abs(sm))
		case 1:
			SetAxis/A=(as) left
			ModifyGraph/W=$cw hideTrace=2
			break
		case 2:
			SetAxis/A=(as) left
			ModifyGraph/W=$cw hideTrace=1
			break
		case 3:
			GetAxis/Q left
			SetAxis left V_min,V_max
			ModifyGraph/W=$cw hideTrace=1
			break
		case 4:
			GetAxis/Q left
			SetAxis left V_min,V_max
			ModifyGraph/W=$cw hideTrace=1
			ModifyGraph/Z/W=$cw hideTrace(scrt_frozen)=0
			break
	endswitch
	theOne = StringFromList(tn,scrtTraceNameList())
	ModifyGraph/W=$cw hideTrace($theOne)=0
	
	return 0
end

// set loop once or multiple times

Function SetLoopMode(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR lm = cwDFR:loopmode
	SVAR pn = cwDFR:panelName

	switch( cba.eventCode )
		case 99:
			CheckBox $cba.ctrlName win=$(cw + "#" + pn), value = 1
		case 2: // mouse up
			strswitch(cba.ctrlName)
				case "once":
					CheckBox loop win=$(cw + "#" + pn), value=0
					lm = 0
					break
				case "loop":
					Checkbox once win=$(cw + "#" + pn), value=0
					lm = 1
					break
			endswitch
			break
	endswitch

	return 0
end

// start loop thru traces

Function DoLoop(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR lm = cwDFR:loopmode
	SVAR pn = cwDFR:panelname
	NVAR tn = cwDFR:tracenumber

	variable fw = 1, st

	// disable movie mode for now!
	
//	return 0
	
	switch( ba.eventCode )
		case 2:
//			print "DoLoop ", tn, mm
			if (!mm)
				strswitch(ba.ctrlName)
					case "LoopForward":
						Button LoopBack win=$(cw + "#" + pn), disable=2
						break
					case "LoopBack":
						Button LoopForward win=$(cw + "#" + pn), disable=2
						fw = 0
						break
				endswitch
				Button $ba.ctrlName win=$(cw + "#" + pn), title="X", userdata="Active", disable=0, noproc
				st = 2
			else
				mm = 0
				DoUpdate/W=$(cw + "#" + pn)/E=0
				SetWindowHooks(0)
				strswitch(ba.ctrlName)
					case "LoopForward":
						Button LoopBack win=$(cw + "#" + pn), disable=0
						Button LoopForward win=$(cw + "#" + pn), title=">>", userdata="", proc=DoLoop
						break
					case "LoopBack":
						Button LoopForward win=$(cw + "#" + pn), disable=0
						Button LoopBack win=$(cw + "#" + pn), title="<<", userdata="", proc=DoLoop
						break
				endswitch
				ba.UserData = ""
				st = 0
			endif
			CheckBox once win=$(cw + "#" + pn), disable=st
			CheckBox loop win=$(cw + "#" + pn), disable=st
			Button first win=$(cw + "#" + pn), disable = st
			Button next win=$(cw + "#" + pn), disable = st
			Button previous win=$(cw + "#" + pn), disable = st
			Button last win=$(cw + "#" + pn), disable = st
			Checkbox scale_a win=$(cw + "#" + pn), disable=st
			Checkbox scale_f win=$(cw + "#" + pn), disable=st
			Checkbox scale_h win=$(cw + "#" + pn), disable=st
			Checkbox scale_c win=$(cw + "#" + pn), disable=st
			Checkbox onoff win=$(cw + "#" + pn), disable=st
			Checkbox keyboard win=$(cw + "#" + pn), disable=st
			PopupMenu external win=$(cw + "#" + pn), disable=st
			Button GetHelp win=$(cw + "#" + pn), disable=st
			Button RemoveMe win=$(cw + "#" + pn), disable=st
			if (st == 2)
				mm = 1
				SetWindowHooks(1)
				if (lm==1)
					do
						LoopThruTraces(ba)
						if (cmpStr(ba.userData,"Abort")==0)
//							print "Aborting Loop ", tn, mm
							DoUpdate/W=$(cw + "#" + pn)/E=0
							ba.eventCode=2
							DoLoop(ba)
							break
						else
							if (fw)
								tn = 0
							else
								tn = inf
							endif
						endif
					while(1)
				else
					LoopThruTraces(ba)
//					print "Ending ", tn, mm
					DoUpdate/W=$(cw + "#" + pn)/E=0
					ba.eventCode=2
					DoLoop(ba)
				endif
			endif
			break
	endswitch
	
	return 0
End

// loop thru traces as a movie

Function LoopThruTraces(ba)
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pn = cwDFR:panelname
	NVAR tn = cwDFR:tracenumber
	NVAR sp = cwDFR:speed

	STRUCT WMSetVariableAction sva
	
	variable nt, ic, ks, st, step=1
	
	nt = ItemsInList(scrtTraceNameList())
	st = tn>=nt ? nt-1 : tn
	sva.eventCode=99
	DoUpdate/W=$(cw + "#" + pn)/E=1
	if (cmpStr(ba.ctrlName,"LoopForward")==0)
		try
			for (ic=st;ic<nt;ic+=1)
			
				// do checks for abort

				if (GetKeyState(0) & 32)
					ba.eventCode=2
					ba.userData = "Abort"
//					print "Escaping ", tn, mm
					break
				endif
				
				// go to trace

				tn = ic
				GoToTraceNumber(sva)
				
				// update panel and check for button press
				
				DoUpdate/W=$(cw + "#" + pn)
//				print "V_flag ", V_flag
				if (V_flag==2)
					ba.eventCode=2
					ba.userData = "Abort"
//					print "button press"
					break
				endif
				
				// sleep
				
				Sleep/S 2/(sp)
			endfor
		catch
			// abort via some other reason
			ba.eventCode=2
			ba.userData = "Abort"
			return 0
		endtry
	else
		try
			for (ic=st;ic>=0;ic+=-1)
			
				// do checks for abort

				if (GetKeyState(0) & 32)
					ba.eventCode=2
					ba.userData = "Abort"
//					print "Escaping ", tn, mm
					break
				endif
				
				// go to trace

				tn = ic
				GoToTraceNumber(sva)
				
				// update panel and check for button press
				
				DoUpdate/W=$(cw + "#" + pn)
//				print "V_flag ", V_flag
				if (V_flag==2)
					ba.eventCode=2
					ba.userData = "Abort"
//					print "button press"
					break
				endif
				
				// sleep
				
				Sleep/S 2/(sp)
			endfor
		catch
			// abort via some other reason
			ba.eventCode=2
			ba.userData = "Abort"
			return 0
		endtry
	endif
	return 0
end

// clean up graph and kill things when panel closed

Function CleanUpandKill()
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pn = cwDFR:panelname

	SetWindowHooks(-1)
	LoadScrollTracesPlugIns(0)
	KillWindow $(cw + "#" + pn)
	KillDataFolder root:Packages:ScrollTraces:$(cw)
	ModifyGraph/W=$cw hideTrace=0
	RemoveFromGraph/Z/W=$cw scrt_frozen
	SetAxis/A/Z left
	return 0
end

// general hook function for graph or panel

Function ScrollTracesHook(sh)
	STRUCT WMWinHookStruct &sh
	
	NVAR/SDFR=$thePackageFolder mm = moviemode

	// global movie mode not handled

	variable hr = 0
	
	if (mm)
		return hr
	endif

	SVAR/SDFR=$thePackageFolder cw = currWin
	
	// activating the graph or panel sets its graph current window
	// kill requests (with active panels) first kill panel
	
//	GetWindow $sh.winName activeSW
//	print "check Hook ", S_Value, sh.eventCode
	
	switch (sh.eventCode)
		case 0:	// activate the panel
			cw = WinName(0,1,1)
			hr = 1
			break
		case 17:
			CleanUpandKill()
			break
	endswitch
	
	return hr
end

// hook function that handles keyboard events in non-movie mode

Function ScrollTracesKeyboardHook(sh)
	STRUCT WMWinHookStruct &sh
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR tn = cwDFR:tracenumber
	NVAR sp = cwDFR:speed
	NVAR kb = cwDFR:keyboard
	SVAR pn = cwDFR:panelname

	STRUCT WMSetVariableAction sva
	STRUCT WMCheckboxAction cba
	
	variable hr = 0
	
	// global movie mode not handled

	if (mm)
		return hr
	endif
	
	// handle only k to reactivate keyboard input
	if (!kb)
		if (sh.keycode==107)
			kb = 1
			Checkbox keyboard,  win=$(cw + "#" + pn), value = 1
			hr = 1
		endif
		return hr
	endif
	
	// keyboard requests of an active graph or panel (non-movie mode)
	
//	GetWindow $sh.winName activeSW
//	print "check Keyboard Hook ", S_Value
	
	if (sh.eventCode == 11)
		//print sh.keycode
		switch(sh.keycode)
			case 31:	// up key
				sp += -1
				if (sp<1)
					sp = 1
				endif
				hr = 1
				break
			case 30:	// down key
				sp += 1
				if (sp>20)
					sp = 20
				endif
				hr = 1
				break
			case 29:	// right key
				sva.eventCode=99
				if (GetKeyState(0) & 4)
					tn = inf
				else
					tn += 1
				endif
				GoToTraceNumber(sva)
				hr = 1
				break
			case 28:	// left key
				sva.eventCode=99
				if (GetKeyState(0) & 4)
					tn = 0
				else
					tn += -1
				endif
				GoToTraceNumber(sva)
				hr = 1
				break
			case 97:	// a key
				cba.ctrlName = "scale_a"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 99:	// c key
				cba.ctrlName = "scale_c"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 102: // f key
				cba.ctrlName = "scale_f"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 104: // h key
				cba.ctrlName = "scale_h"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 107: // k key
				kb = 0
				Checkbox keyboard, win=$(cw + "#" + pn), value=0
				hr = 1
				break
			case 114: // r key
				cba.ctrlName = "scale_r"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 111: // o key
				cba.ctrlName = "once"
				cba.eventCode = 99
				SetLoopMode(cba)
				hr = 1
				break
			case 108: // l key
				cba.ctrlName = "loop"
				cba.eventCode = 99
				SetLoopMode(cba)
				hr = 1
				break
		endswitch
	endif
	
	return hr
end

// hook function that handles movie mode

Function ScrollTracesMovieModeHook(sh)
	STRUCT WMWinHookStruct &sh
	
	variable hr = 0

	// only movie mode handled here (for both graph and panel)
	// prevents killing of the active graph and panel
	
//	print "watching movie ", sh.eventCode
	if (sh.eventCode == 17)
		hr = 2
	endif
	
	return hr
end
